<?php

declare(strict_types=1);

namespace CreoCode\ContaoTourismBundle\EventListener;

use Contao\CoreBundle\Framework\ContaoFramework;
use Contao\StringUtil;
use Contao\Framework;

use CreoCode\ContaoTourismBundle\Classes\Accommodations;
use CreoCode\ContaoTourismBundle\Models\AccommodationsModel;

use CreoCode\ContaoTourismBundle\Classes\Offers;
use CreoCode\ContaoTourismBundle\Models\OffersModel;

use CreoCode\ContaoTourismBundle\Classes\Prices;
use CreoCode\ContaoTourismBundle\Models\PricesModel;
use CreoCode\ContaoTourismBundle\Modules\ModulePricesList;

class InsertTagsListener
{
    /**
     * @var ContaoFramework
     */
    private $framework;

    public function __construct(ContaoFramework $framework)
    {
        $this->framework = $framework;
    }

    /**
     * @return string|false
     */
    public function onReplaceInsertTags(string $tag, bool $useCache, $cacheValue, array $flags)
    {
        $elements = explode('::', $tag);
        $key = strtolower($elements[0]);

        static $supportedTagsAccommodations = [
            'accommodations',
            'accommodations_open',
            'accommodations_url',
            'accommodations_title',
            'accommodations_teaser',
        ];

        if (\in_array($key, $supportedTagsAccommodations, true)) {
            return $this->replaceAccommodationsInsertTags($key, $elements[1], $flags);
        }


        static $supportedTagsOffers = [
            'offers',
            'offers_open',
            'offers_url',
            'offers_title',
            'offers_teaser',
        ];

        if (\in_array($key, $supportedTagsOffers, true)) {
            return $this->replaceOffersInsertTags($key, $elements[1], $flags);
        }


        static $supportedTagsPrices = [
            'prices',
        ];

        if (\in_array($key, $supportedTagsPrices, true)) {
            return $this->replacePricesInsertTags($tag, $flags);
        }


        return false;
    }

    private function replaceAccommodationsInsertTags(string $insertTag, string $idOrAlias, array $flags): string
    {
        $this->framework->initialize();

        /** @var AccommodationsModel $adapter */
        $adapter = $this->framework->getAdapter(AccommodationsModel::class);

        if (null === ($model = $adapter->findByIdOrAlias($idOrAlias))) {
            return '';
        }

        /** @var Accommodations $accommodations */
        $accommodations = $this->framework->getAdapter(Accommodations::class);

        switch ($insertTag) {
            case 'accommodations':
                return sprintf(
                    '<a href="%s" title="%s">%s</a>',
                    $accommodations->generateAccommodationsUrl($model, false, \in_array('absolute', $flags, true)),
                    StringUtil::specialchars($model->headline),
                    $model->headline
                );

            case 'accommodations_open':
                return sprintf(
                    '<a href="%s" title="%s">',
                    $accommodations->generateAccommodationsUrl($model, false, \in_array('absolute', $flags, true)),
                    StringUtil::specialchars($model->headline)
                );

            case 'accommodations_url':
                return $accommodations->generateAccommodationsUrl($model, false, \in_array('absolute', $flags, true));

            case 'accommodations_title':
                return StringUtil::specialchars($model->headline);

            case 'accommodations_teaser':
                return StringUtil::toHtml5($model->teaser);
        }

        return '';
    }

    private function replaceOffersInsertTags(string $insertTag, string $idOrAlias, array $flags): string
    {
        $this->framework->initialize();

        /** @var OffersModel $adapter */
        $adapter = $this->framework->getAdapter(OffersModel::class);

        if (null === ($model = $adapter->findByIdOrAlias($idOrAlias))) {
            return '';
        }

        /** @var Offers $offers */
        $offers = $this->framework->getAdapter(Offers::class);

        switch ($insertTag) {
            case 'offers':
                return sprintf(
                    '<a href="%s" title="%s">%s</a>',
                    $offers->generateOffersUrl($model, false, \in_array('absolute', $flags, true)),
                    StringUtil::specialchars($model->headline),
                    $model->headline
                );

            case 'offers_open':
                return sprintf(
                    '<a href="%s" title="%s">',
                    $offers->generateOffersUrl($model, false, \in_array('absolute', $flags, true)),
                    StringUtil::specialchars($model->headline)
                );

            case 'offers_url':
                return $offers->generateOffersUrl($model, false, \in_array('absolute', $flags, true));

            case 'offers_title':
                return StringUtil::specialchars($model->headline);

            case 'offers_teaser':
                return StringUtil::toHtml5($model->teaser);
        }

        return '';
    }

    private function replacePricesInsertTags(string $insertTag, array $flags): string
    {

        $chunks = explode('::', $insertTag);

        $key = $chunks[0];
        $idOrAlias = $chunks[1];
        $subIdOrAlias = $chunks[2];

        $this->framework->initialize();


        /** @var PricesModel $adapter */
        $adapter = $this->framework->getAdapter(PricesModel::class);

        if (null === ($model = $adapter->findByIdOrAlias($idOrAlias))) {
            return '';
        }

        /** @var Prices $prices */
        $prices = $this->framework->getAdapter(Prices::class);

        switch ($key) {
            case 'prices':

                // Add the articles
                if ($chunks[1] !== null)
                {

                    $objModuleModel = new \ModuleModel();
                    $objModuleModel->type = 'priceslist';
                    $objModuleModel->id = uniqid();
                    $objModuleModel->prices_template = 'prices_table';
                    $objModuleModel->prices_sets = serialize([$chunks[1]]);

                    if ($chunks[2] !== null) {
                        $objModuleModel->prices_keys = serialize([$chunks[1].'__'.$chunks[2]]);
                    }

                    // generate the navigation module
                    $objModule = new ModulePricesList($objModuleModel);

                    // generate the html and return it
                    return $objModule->generate();
                }
        }

        return '';
    }

}



