<?php

namespace CreoCode\ContaoTourismBundle\Modules;

use Contao\Model\Collection;
use FOS\HttpCache\ResponseTagger;

use Contao\Module;
use Contao\FrontendUser;
use Contao\FrontendTemplate;
use Contao\StringUtil;
use Contao\System;
use Contao\ContentModel;
use Contao\FilesModel;

use CreoCode\ContaoTourismBundle\Classes\Prices;
use CreoCode\ContaoTourismBundle\Models\PricesSetModel;
use CreoCode\ContaoTourismBundle\Models\PricesModel;
use CreoCode\ContaoTourismBundle\Models\PricesTableModel;

/**
 * Parent class for prices modules.
 *
 * @property string $prices_template
 * @property mixed  $prices_metaFields
 *
 */
abstract class ModulePrices extends Module
{

	/**
	 * Sort out protected sets
	 *
	 * @param array $arrSets
	 *
	 * @return array
	 */
	protected function sortOutProtected($arrSets)
	{
		if (empty($arrSets) || !\is_array($arrSets))
		{
			return $arrSets;
		}

		$this->import(FrontendUser::class, 'User');
		$objSet = PricesSetModel::findMultipleByIds($arrSets);
		$arrSets = array();


		if ($objSet !== null)
		{
			while ($objSet->next())
			{
				if ($objSet->protected)
				{
					if (!FE_USER_LOGGED_IN || !\is_array($this->User->groups))
					{
						continue;
					}

					$groups = StringUtil::deserialize($objSet->groups);

					if (empty($groups) || !\is_array($groups) || !\count(array_intersect($groups, $this->User->groups)))
					{
						continue;
					}
				}

				$arrSets[] = $objSet->id;
			}
		}

		return $arrSets;
	}

	/**
	 * Parse an item and return it as string
	 *
	 * @param PricesModel $objArticle
	 * @param boolean   $blnAddSet
	 * @param string    $strClass
	 * @param integer   $intCount
	 *
	 * @return string
	 */
	protected function parseArticle($objArticle, $blnAddSet=false, $strClass='', $intCount=0)
	{
		$objTemplate = new FrontendTemplate($this->prices_template);
		$objTemplate->setData($objArticle->row());

		if ($objArticle->cssClass != '')
		{
			$strClass = ' ' . $objArticle->cssClass . $strClass;
		}

		if ($objArticle->featured)
		{
			$strClass = ' featured' . $strClass;
		}

		$objTemplate->class = $strClass;
		$objTemplate->pricesHeadline = $objArticle->headline;
		$objTemplate->set = $objArticle->getRelated('pid');
		$objTemplate->count = $intCount; // see #5708
		$objTemplate->text = '';
		$objTemplate->currency = $this->currency;
		$objTemplate->currencySymbol = $this->convertCurrencyToSymbol($this->currency);

		$objTemplate->tablesHeader = array();
		$objTemplate->tables = array();


		$objTablesHeader = $this->fetchPriceTablesHeader([$objArticle->id]);
		if ($objTablesHeader !== null) {
			$objTemplate->tablesHeader = unserialize($objTablesHeader);
		}

		$objTables = $this->fetchPriceTables([$objArticle->id], $blnFeatured, 0, 0);

		// Add the articles
		if ($objTables !== null)
		{
			// $this->Template->articles = $this->parseArticles($objTables);
			$objTemplate->tables = $objTables;
		}

		// HOOK: add custom logic
		if (isset($GLOBALS['TL_HOOKS']['parseArticles']) && \is_array($GLOBALS['TL_HOOKS']['parseArticles']))
		{
			foreach ($GLOBALS['TL_HOOKS']['parseArticles'] as $callback)
			{
				$this->import($callback[0]);
				$this->{$callback[0]}->{$callback[1]}($objTemplate, $objArticle->row(), $this);
			}
		}

		// Tag the response
		if (System::getContainer()->has('fos_http_cache.http.symfony_response_tagger'))
		{
			/** @var ResponseTagger $responseTagger */
			$responseTagger = System::getContainer()->get('fos_http_cache.http.symfony_response_tagger');
			$responseTagger->addTags(array('contao.db.tl_prices.' . $objArticle->id));
			$responseTagger->addTags(array('contao.db.tl_prices_set.' . $objArticle->pid));
		}

		return $objTemplate->parse();
	}

	/**
	 * Parse one or more items and return them as array
	 *
	 * @param Collection $objArticles
	 * @param boolean    $blnAddSet
	 *
	 * @return array
	 */
	protected function parseArticles($objArticles, $blnAddSet=false)
	{

		$limit = $objArticles->count();

		if ($limit < 1)
		{
			return array();
		}

		$count = 0;
		$arrArticles = array();

		while ($objArticles->next())
		{
			/** @var PricesModel $objArticle */
			$objArticle = $objArticles->current();

			$arrArticles[] = $this->parseArticle($objArticle, $blnAddSet, ((++$count == 1) ? ' first' : '') . (($count == $limit) ? ' last' : '') . ((($count % 2) == 0) ? ' odd' : ' even'), $count);
		}

		return $arrArticles;
	}
	
	/**
	 * Converts currency to symbol
	 *
	 * @return string
	 */
	protected function convertCurrencyToSymbol($currency)
	{
		$currencySymbol = $GLOBALS['TL_LANG']['tl_tourism']['currency_symbol']['EUR'];
		return $currencySymbol;
	}


	/**
	 * Fetch the matching items
	 *
	 * @param array   $pricesSetsPid
	 *
	 * @return Collection|PricesModel|null
	 */
	protected function fetchPriceTablesHeader($pid)
	{
		$t = PricesModel::getTable();
		$prices = PricesModel::findById($pid);

		return $prices->config;
	}

	/**
	 * Fetch the matching items
	 *
	 * @param array   $pricesSets
	 * @param boolean $blnFeatured
	 * @param integer $limit
	 * @param integer $offset
	 *
	 * @return Collection|PricesModel|null
	 */
	protected function fetchPriceTables($pricesSets, $blnFeatured, $limit, $offset)
	{

		// Determine sorting
		$t = PricesTableModel::getTable();
		$arrOptions = array();

		switch ($this->prices_order)
		{
			case 'order_sorting_asc':
				$arrOptions['order'] = "$t.sorting";
				break;

			case 'order_sorting_desc':
				$arrOptions['order'] = "$t.sorting DESC";
				break;
				
			case 'order_headline_asc':
				$arrOptions['order'] = "$t.headline";
				break;

			case 'order_headline_desc':
				$arrOptions['order'] = "$t.headline DESC";
				break;

			case 'order_random':
				$arrOptions['order'] = "RAND()";
				break;

			default:
				$arrOptions['order'] = "$t.sorting";
		}

		if($this->prices_keys) {
			$arrOptions['keys'] = unserialize($this->prices_keys);
		}


		// return PricesTableModel::findPublishedByPids($pricesSets, $blnFeatured, $limit, $offset, $arrOptions);
		return PricesTableModel::findPublishedByPidsAndKeys($pricesSets, $blnFeatured, $limit, $offset, $arrOptions);
	}
}

class_alias(ModulePrices::class, 'ModulePrices');
